/* === NEW: Styles for the Sync Task List === */
.sync-task-card {
    transition: box-shadow 0.2s ease-in-out;
}

.sync-task-card:hover {
    box-shadow: 0 0.5rem 1rem rgba(0, 0, 0, 0.15);
}

.sync-task-card .card-body {
    padding: 1rem;
}

.sync-task-card .task-status-icon {
    font-size: 1.5rem;
}

.sync-task-card .task-name {
    font-weight: 600;
}

.sync-task-card .task-details {
    font-size: 0.85rem;
    color: #6c757d;
}

.sync-task-card .task-actions .btn {
    padding: 0.25rem 0.5rem;
    font-size: 0.75rem;
}